extern  char *flook(char *fname,int hflag);
extern  char *keystrings(int key);
extern  char addline(char *text);
extern  char anycb(void );
extern  char asciimode(void );
extern  char autosave(void );
extern  char backchar(int f,int n,int k);
extern  char backdel(int f,int n,int k);
extern  char backline(int f,int n,int k);
extern  char backpage(int f,int n,int k);
extern  char backsearch(void );
extern  char backsrch(void );
extern  char backunit(int f,int n,int k);
extern  char bclear(struct BUFFER *bp);
extern  char binarymode(void );
extern  char bindtokey(void );
extern  char buffername(void );
extern  char bufsizlock(void );
extern  char b_append_c(struct BUFFER *buf_p,unsigned char ch);
extern  char compare(void );
extern  char copyregion(int f,int n,int k);
extern  char ctlxe(int f,int n,int k);
extern  char ctlxlp(int f,int n,int k);
extern  char ctlxrp(int f,int n,int k);
extern  char ctrlg(int f,int n,int k);
extern  char decimalmode(void );
extern  char dec_chr_ok(char *char_buf,char *max_str,char chr,char pos);
extern  char delbunit(int f,int n,int k);
extern  char delfunit(int f,int n,int k);
extern  char delwind(void );
extern  char dispshift(int f,int n,int k);
extern  char dispsize1(void );
extern  char dispsize2(void );
extern  char dispsize4(void );
extern  char dispswapbyte(int f,int n,int k);
extern  char ebcdicmode(void );
extern  char enlargewind(int f,int n,int k);
extern  char eread(char *fp,char *buf,int nbuf,int flag,char *ap);
extern  char ereply(char *fp,char *buf,int nbuf,char *arg);
extern  char execute(int c,int f,int n);
extern  char extend(int f,int n,int k);
extern  char eyesno(char *sp);
extern  char fbackupfile(char *fname);
extern  char ffclose(void );
extern  char ffgetline(char *buf,unsigned int nbuf,unsigned int *rbuf);
extern  char ffputline(char *buf,int nbuf);
extern  char ffropen(char *fn);
extern  char ffwopen(char *fn,unsigned short mode);
extern  char fileinsert(void );
extern  char filename(void );
extern  char fileread(void );
extern  char filesave(void );
extern  char filevisit(void );
extern  char filewrite(void );
extern  char file_visit(int f,int n,int k);
extern  char fill_out(void );
extern  char flushnquit(int f,int n,int k);
extern  char flush_all(void );
extern  char forwchar(int f,int n,int k);
extern  char forwdel(int f,int n,int k);
extern  char forwline(int f,int n,int k);
extern  char forwpage(int f,int n,int k);
extern  char forwsearch(void );
extern  char forwsrch(void );
extern  char forwunit(int f,int n,int k);
extern  char getregion(struct reg *rp);
extern  char getscreenstate(void );
extern  char gotobob(void );
extern  char gotoeob(void );
extern  char gotoline(int f,int n,int k);
extern  char help(void );
extern  char hexmode(void );
extern  char insertunit(int f,int n,int k);
extern  char insert_toggle(void );
extern  char killablebufs(struct BUFFER *bp);
extern  char killbuffer(void );
extern  char killregion(int f,int n,int k);
extern  char ldelete(unsigned long n_bytes,int kflag);
extern  char linkwind(void );
extern  char linsert(int n,unsigned char c);
extern  char listbuffers(void );
extern  char load_extend(void );
extern  char load_file(char *fname,unsigned long start,unsigned long end);
extern  char lrepl_str(int plen,struct LINE *rstr,struct LINE *mstr);
extern  char makelist(void );
extern  char move_ptr(struct WINDOW *wp,long len,char dot,char fix,char rel);
extern  char mvdnwind(int f,int n,int k);
extern  char mvupwind(int f,int n,int k);
extern  char nextwind(void );
extern  char next_buf(void );
extern  char n_way_combine(int f,int n,int k);
extern  char n_way_split(int f,int n,int k);
extern  char octalmode(void );
extern  char onebuf(struct BUFFER *bp);
extern  char onlywind(void );
extern  char parse_f_name(char *fn,unsigned long *start,unsigned long *end);
extern  char pickone(void );
extern  char popblist(void );
extern  char prevwind(void );
extern  char prev_buf(void );
extern  char print(void );
extern  char queryrepl(int f,int n,int k);
extern  char quit(int f,int n,int k);
extern  char quote(int f,int n,int k);
extern  char readin(char *fname,unsigned long start,unsigned long end);
extern  char readpattern(void );
extern  char recall(void );
extern  char refresh(void );
extern  char replaceit(void );
extern  char reposition(void );
extern  char save_region(int f,int n,int k);
extern  char searchagain(void );
extern  char selfinsert(int f,int n,int k);
extern  char setmark(void );
extern  char showcpos(int f,int n,int k);
extern  char showsavebuf(void );
extern  char showversion(int f,int n,int k);
extern  char shrinkwind(int f,int n,int k);
extern  char spawncli(int f,int n,int k);
extern  char splitwind(void );
extern  char swapmark(void );
extern  char to_ebcdic(char ch);
extern  char twiddle(void );
extern  char usebuffer(void );
extern  char use_buffer(void );
extern  char viewfile(void );
extern  char vtputd(struct WINDOW *wp,int row);
extern  char wallchart(int f,int n,int k);
extern  char wind_on_dot(struct WINDOW *wp);
extern  char writeout(char *fn,unsigned long start,unsigned long end,unsigned short mode);
extern  char yank(int f,int n,int k);
extern  char yank_buffer(void );
extern  char _killbuffer(char *bufn);
extern  char _usebuffer(char *bufn);
extern  char _yankbuffer(char *bufn);
extern  int find_keyval(char *name);
extern  int getctl(void );
extern  int getkbd(void );
extern  int getkey(void );
extern  int getsysconfig(union REGS *outregs,struct SREGS *segregs);
extern  int getxtra(struct SYMBOL *sp1,struct SYMBOL *sp2,int cpos);
extern  int symhash(char *cp);
extern  int ttgetc(void );
extern  int ttkeyready(void );
extern  struct BUFFER *bcreate(char *bname);
extern  struct BUFFER *bfind(char *bname,int cflag);
extern  struct LINE *lalloc(int size);
extern  struct LINE *l_break_in_two(struct LINE *lp,unsigned int lo,unsigned int extra);
extern  struct SYMBOL *symlookup(char *cp);
extern  struct WINDOW *wpopup(void );
extern  unsigned int fill_buf(struct WINDOW *wp,struct LINE *lin,unsigned int lin_off,char *w_buff,unsigned int cnt);
extern  unsigned int get_curcol(struct WINDOW *wp);
extern  unsigned int get_currow(struct WINDOW *wp);
extern  unsigned long ffseek(unsigned long posn);
extern  unsigned long file_len(void );
extern  unsigned long get_long(unsigned char *w_buf);
extern  unsigned short get_int(unsigned char *w_buf);
extern  unsigned short get_save_char(void );
extern  void adjustcase(char *fn);
extern  void asciiparm(int n);
extern  void bad_key(int key);
extern  void bin_to_text(char *bin_buf,char *txt_buf,unsigned int len,struct ROW_FMT *fmt_ptr);
extern  void b_append_l(struct BUFFER *buf_p,struct LINE *lp);
extern  void check_extend(char *sfname);
extern  void edinit(char *bname);
extern  void eerase(void );
extern  void eformat(char *fp,char *ap);
extern  void eputc(int c);
extern  void eputi(int i,int r);
extern  void eputs(char *s);
extern  void err_echo(char *buf);
extern  void funky_name(char *bname,int n);
extern  void is_wang(void );
extern  void keyadd(int new,char (*funcp)(),char *name,int modify);
extern  void keydup(int new,char *name);
extern  void keymapinit(void );
extern  void keyname(char *cp,int k);
extern  void lchange(int flag);
extern  void lfree(struct LINE *lp);
extern  void lreplace(int n,char c);
extern  void l_fix_up(struct LINE *line);
extern  void main(int argc,char * *argv);
extern  void makename(char *bname,char *fname);
extern  void mem_line(int row,struct vid *vvp);
extern  void modeline(struct WINDOW *wp);
extern  void next_pat(void );
extern  void putline(int row,int startcol,int stringsize,char *string);
extern  void save_buf_home(void );
extern  void save_buf_init(void );
extern  void set_crt_type(void );
extern  void set_mode_vars(void );
extern  void sort_buf(struct BUFFER *b_ptr,int cnt);
extern  void ttbeep(void );
extern  void ttclose(void );
extern  void ttcolor(int color);
extern  void ttcooked(void );
extern  void tteeol(void );
extern  void tteeop(void );
extern  void ttflush(void );
extern  void ttinit(void );
extern  void ttmove(int row,int col);
extern  void ttnowindow(void );
extern  void ttopen(void );
extern  void ttputc(int c);
extern  void ttraw(void );
extern  void tttidy(void );
extern  void ttykeymapinit(void );
extern  void ucopy(struct vid *vvp,struct vid *pvp);
extern  void uline(int row,struct vid *vvp,struct vid *pvp);
extern  void ungetkey(int k);
extern  void update(void );
extern  void vteeol(void );
extern  void vtinit(void );
extern  void vtmove(int row,int col);
extern  void vtputc(char c);
extern  void vttidy(void );
extern  void wind_on_dot_all(void );
extern  void writ_echo(char *buf);
extern  void _lowercase(char *s);
